<?php

namespace App\Http\Controllers;

use App\Mail\TaskAssignedMail;
use App\Models\Task;
use App\Models\Project;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;
use App\Notifications\TaskAssignedNotification;
use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Facades\Log;
use App\Jobs\SendTaskAssignedMail;
use App\Jobs\SendTaskNotification;

class TaskController extends Controller
{
    public function index()
    {
        $tasks = Task::with(['project','creator','files'])->get(); 
        $projects = Project::all();
        $users = User::all(); 

        return view('admin.tasks', compact('tasks','projects', 'users'));
    }

    public function store(Request $request)
    {
        $validated = $request->validate([
            'project_id' => 'required|exists:projects,id',
            'title' => 'required|string|max:255|unique:tasks,title',
            'description' => 'required|string',
            'status' => 'required|string|max:255',
            'priority' => 'required|string|max:255',
            'due_date' => 'required|date|after_or_equal:today',
            'created_by' => 'required|exists:users,id',
        ]);

        $task=Task::create($validated);
        return redirect()->route('admin.tasks')->with('success', 'Task created successfully.');
    }

    public function assignUsers(Request $request, Task $task)
    {
        $validated = $request->validate([
            'assigned_users' => 'required|array',
            'assigned_users.*' => 'exists:users,id',
        ]);

        $task->assignedUsers()->syncWithPivotValues($validated['assigned_users'], [
            'assigned_by' => auth()->id(),
            'created_at' => now(),
            'updated_at' => now()
        ]);

        foreach ($validated['assigned_users'] as $userId) {
            $user = User::find($userId);
            
            if (!$user) {
                Log::error("User with ID {$userId} not found");
                continue;
            }
            dispatch(new SendTaskAssignedMail($task, $user));
            dispatch(new SendTaskNotification($task, $user, auth()->user()->name));
    }
    }


    public function taskList()
    {
        $tasks = Task::with(['project', 'creator', 'assignedUsers', 'files.uploader'])->get();
        return view('admin.task-list', compact('tasks'));
    }







    public function update(Request $request, $id)
    {
        $task = Task::findOrFail($id);

        $request->validate([
            'project_id' => 'required|exists:projects,id',
            'title' => 'required|string|max:255|unique:tasks,title,' . $id,
            'description' => 'required|string',
            'status' => 'required|string|max:255',
            'priority' => 'required|string|max:255',
            'due_date' => 'required|date',
            'assigned_users' => 'nullable|array',
            'assigned_users.*' => 'exists:users,id',
        ]);

        $task->update($request->only(['project_id', 'title', 'description', 'status', 'priority', 'due_date']));

        if ($request->has('assigned_users')) {
            $task->assignedUsers()->syncWithPivotValues($request->assigned_users, [
                'assigned_by' => auth()->id(),
                'created_at' => now(),
                'updated_at' => now()
            ]);

            foreach ($request->assigned_users as $userId) {
                $user = User::find($userId);

                if (!$user) {
                    Log::error("User with ID {$userId} not found");
                    continue;
                }
            dispatch(new SendTaskAssignedMail($task, $user));
            dispatch(new SendTaskNotification($task, $user, auth()->user()->name));
        }
    }

    return redirect()->route('admin.tasks')->with('success', 'Task updated and notifications queued.');
}

public function delete($id)
{
    $task = Task::findOrFail($id);
    $task->delete();

    return redirect()->route('admin.tasks')->with('success', 'Task deleted successfully.');
}


}
